/*
 Wireless Gameboy Controller RX
 Version: 1.1
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 23/09/2018
 Last Modified: 26/10/2018
 
 Use your Gameboy, GBC, GBP, GBA or GBA SP as a Wireless game controller for any device which supports USB HID Joysticks.
 This would allow you to play games on your PC with your a Gameboy of your choosing.
 
 
 Program the ATtiny841 to use a 12MHz crystal
 avrdude -p ATtiny841 -c usbasp -U lfuse:w:0xff:m -U hfuse:w:0xdf:m -U efuse:w:0xff:m
 
*/

// ATtiny841 Pin map
//
//                     +-\/-+
//               VCC  1|o   |14  GND
//    12MHz XTAL PB0  2|    |13  PA0 USB D+
//    12MHz XTAL PB1  3|    |12  PA1 USB D-
//         Reset PB3  4|    |11  PA2 LED
//        USB D+ PB2  5|    |10  PA3 nRF24L01 CE
//  nRF24L01 CSN PA7  6|    |9   PA4 nRF24L01 SCK
//   nRF24L01 DI PA6  7|    |8   PA5 nRF24L01 DO
//                     +----+


#define F_CPU 12000000 // If this is changed, modify it in usbconfig.h too

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <math.h>
#include <util/delay.h>
#include "usbdrv/usbdrv.h"
#include "usbdrv/oddebug.h"
#include "setup.c"
#include "mirf.c"


uint8_t kpData = 0;

struct{
  union {
    uint8_t data;
    struct {
      uint8_t X:2;
      uint8_t Y:2;
      uint8_t B:1;
      uint8_t A:1;
      uint8_t SELECT:1;
      uint8_t START:1;
    };
  };
} reportBuffer;


int main (void) {
	setup();
	usbInit(); // Start USB
	
	check_for_new_settings(); // Check for new settings for 1 second
	mirf_config();
	RX_POWERUP;
	mirf_flush_rx_tx(); // Flush TX/RX
	mirf_CE_hi; // Start listening
	
	while(1) {
		if (mirf_status() & (1<<RX_DR)) {
			PORTA |= (1<<LED);
			mirf_CE_lo; // Stop listening
			
			mirf_CSN_lo; // Pull down chip select
			spi_transfer(R_RX_PAYLOAD); // Send cmd to read rx payload
			kpData = spi_transfer(0); // Read data
			mirf_CSN_hi; // Pull up chip select
			mirf_config_register(STATUS,(1<<RX_DR)); // Reset status register
			
			mirf_flush_rx_tx(); // Flush TX/RX
			mirf_CE_hi; // Start listening
			PORTA &= ~(1<<LED);
		}
		
		usbPoll();
		
		if (usbInterruptIsReady()) {
			reportBuffer.data = 0x05; // Center pad, little endian
			
			// Check for key presses
			if (kpData & 0x01) { // A
				reportBuffer.A = 1;
			}
			if (kpData & 0x02) { // B
				reportBuffer.B = 1;
			}
			if (kpData & 0x04) { // Right
				reportBuffer.X++;
			}
			if (kpData & 0x08) { // Left
				reportBuffer.X--;
			}
			if (kpData & 0x10) { // Down
				reportBuffer.Y--;
			}
			if (kpData & 0x20) { // Up
				reportBuffer.Y++;
			}
			if (kpData & 0x40) { // Select
				reportBuffer.SELECT = 1;
			}
			if (kpData & 0x80) { // Start
				reportBuffer.START = 1;
			}
			
			// Called after every poll of the interrupt endpoint
			usbSetInterrupt(&reportBuffer, sizeof(reportBuffer));
		}
	}
	
	return 0;
}
